﻿using Opc.Ua;
using OpcLabs.BaseLib.Instrumentation;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.Application;
using OpcLabs.EasyOpc.UA.Engine;
using OpcLabs.EasyOpc.UA.OperationModel;

class Program
{
   static void Main()
   {
      UAEndpointDescriptor endpointDescriptor = "opc.tcp://thederw10.bauercontrols.local:49321";
      endpointDescriptor.EndpointSelectionPolicy = new UAEndpointSelectionPolicy();
      var client = new EasyUAClient();
      EasyUAApplication application = new EasyUAApplication();

      // Hook static events
      EasyUAClient.LogEntry += EasyUAClientOnLogEntry;
      try
      {
         // Set the application name, which determines the subject of the client certificate.
         // Note that this only works once in each host process
         // Do something - invoke an OPC read, to trigger some loggable entries.
         try
         {
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AllowUserAcceptCertificate = false;
            EasyUAClient.SharedParameters.EngineParameters.CertificateAcceptancePolicy.AcceptAnyCertificate = true;
            EasyUAApplication.Instance.ApplicationParameters.ApplicationManifest.InstanceOwnStorePath = "C:/TestingMyOwnCertStore";
            endpointDescriptor.EndpointSelectionPolicy = UAMessageSecurityModes.SecuritySignAndEncrypt;
            endpointDescriptor.EndpointSelectionPolicy.SecurityPolicyUriString = UASecurityPolicyUriStrings.Basic256Sha256;

            endpointDescriptor.UserIdentity.UserNameTokenInfo.UserName = "BauerClient";
            endpointDescriptor.UserIdentity.UserNameTokenInfo.Password = "admin@bauercontrols";
            // client.InstanceParameters.
            var attribute = client.Read(endpointDescriptor, "nsu=KEPServerEX ;ns=2;s=Channel1.Device1.Tag1");

            Console.WriteLine($"Tag Value :{0}", attribute.Value);
         }
         catch (UAException uaException)
         {
            Console.WriteLine("*** Failure: {0}", uaException.GetBaseException().Message);
         }


         Console.WriteLine("Processing log entry events for 10 seconds...");
         System.Threading.Thread.Sleep(10 * 1000);

         Console.WriteLine("Finished.");
      }
      finally
      {
         // Unhook static events
         EasyUAClient.LogEntry -= EasyUAClientOnLogEntry;
      }
   }


   private static void EasyUAClientOnLogEntry(object sender, LogEntryEventArgs logEntryEventArgs)
   {
      if (logEntryEventArgs.EventId == 161)
         Console.WriteLine(logEntryEventArgs);
   }
}
